@@ -1,66 +1,69 @@
+import asyncio
 import io
 import json
 import os.path
-import random
 import time
+from typing import List, Union
 
 import grpc
+import pandas
 from github.com.metaprov.modelaapi.pkg.apis.data.v1alpha1.generated_pb2 import Dataset as MDDataset
-from github.com.metaprov.modelaapi.services.dataset.v1.dataset_pb2_grpc import DatasetServiceStub
 from github.com.metaprov.modelaapi.services.dataset.v1.dataset_pb2 import CreateDatasetRequest, \
     UpdateDatasetRequest, \
     DeleteDatasetRequest, GetDatasetRequest, ListDatasetsRequest, GetDatasetProfileRequest
+from github.com.metaprov.modelaapi.services.dataset.v1.dataset_pb2_grpc import DatasetServiceStub
 from tabulate import tabulate
-from tqdm import tqdm, trange
+from tqdm import tqdm
 
-from modela.Resource import Resource
 from modela.ModelaException import ModelaException
-from typing import List, Union
-import pandas
-
-from modela.data.common import *
+from modela.Resource import Resource
+from modela.common import ObjectReference
 from modela.data.DataSource import DataSource
+from modela.data.common import *
 from modela.data.models import SampleSettings, DatasetSpec, DatasetStatus, DatasetProfile
-from modela.infra.models import Workload, NotificationSetting
+from modela.infra.Lab import Lab
+from modela.infra.VirtualBucket import VirtualBucket
+from modela.infra.models import Workload, NotificationSettings
 from modela.training.Report import Report
 from modela.training.common import TaskType
 from modela.util import convert_size
 
 
 class Dataset(Resource):
-    """
-    Testing dataset resource
-    """
-
     def __init__(self, item: MDDataset = MDDataset(), client=None, namespace="", name="",
                  version=Resource.DefaultVersion,
+                 lab: Union[ObjectReference, Lab, str] = "modela-lab",
                  gen_datasource: bool = False,
                  target_column: str = None,
                  datasource: Union[DataSource, str] = "",
-                 bucket: str = "default-minio-bucket",
+                 bucket: Union[VirtualBucket, str] = "default-minio-bucket",
                  dataframe: pandas.DataFrame = None,
                  data_file: str = None,
                  data_bytes: bytes = None,
                  workload: Workload = Workload("general-large"),
-                 sample=SampleSettings(),
-                 task_type: TaskType = TaskType.BinaryClassification,
-                 notification: NotificationSetting = None):
+                 fast: bool = False,
+                 sample: SampleSettings = SampleSettings(),
+                 task_type: TaskType = None,
+                 notification: NotificationSettings = None):
         """
         :param client: The Dataset client repository, which can be obtained through an instance of Modela.
         :param namespace: The target namespace of the resource.
         :param name: The name of the resource.
+        :param lab: The object reference, Lab object, or lab name under the tenant of the resource for which all
+            Dataset-related workloads will be performed under.
         :param gen_datasource: If true, a Datasource resource will be created from the uploaded dataset and applied to
             the Dataset resource.
         :param target_column: If gen_datasource is enabled, then the target column of the data source must be specified.
         :param datasource: If specified as a string, the SDK will attempt to find a Data Source resource with the given name.
             If specified as a Data Source, or if one was found with the given name, it will be applied to the Dataset.
         :param bucket: The bucket which the raw dataset data will be uploaded to.
-        :param dataframe: If specified, the Pandas Dataframe will be serialized and uploaded for ingestion with the Dataset resource.
-        :param data_file: If specified, the SDK will attempt read a file with the given path and will upload the
-            contents of the file for ingestion with the Dataset resource.
-        :param data_bytes: If specified, the SDK will upload the given raw data for ingestion with the Dataset resource.
+        :param dataframe: The Pandas Dataframe will be serialized and uploaded for ingestion with the Dataset resource.
+        :param data_file: The file path which will be read and uploaded for ingestion with the Dataset resource.
+        :param data_bytes: The raw data as bytes that will be uploaded for  ingestion with the Dataset resource.
         :param workload: The resource requirements which will be allocated for Dataset ingestion.
-        :param sample: The sample settings of the dataset, which if enabled will ingest a Dataset with a portion of the uploaded data.
+        :param fast: If enabled, the Dataset will skip validation, profiling, and reporting.
+        :param sample: The sample settings of the dataset, which if enabled will select only a portion of the Dataset
+          for further processing.
         :param task_type: The target task type in relation to the data being used.
         :param notification: The notification settings, which if enabled will forward events about this resource to a notifier.
         """
@@ -70,13 +73,25 @@ class Dataset(Resource):
         if not self.default_resource:  # Ignore the rest of the constructor; datasets are immutable
             return
 
-        if not gen_datasource:
-            if type(datasource) != DataSource:  # Fetch the data source in case we need to read the file type
-                datasource = client.modela.DataSource(namespace=namespace, name=datasource)
+        spec = self.spec
+        if type(lab) == Lab:
+            lab = lab.reference
+        elif type(lab) == str:
+            lab = ObjectReference(Namespace=client.modela.tenant, Name=lab)
+        spec.LabRef = lab
 
-            file_type = datasource.spec.FileType
-        else:
+        if type(bucket) == VirtualBucket:
+            bucket = bucket.name
+
+        if gen_datasource:
             file_type = FlatFileType.Csv
+        else:
+            if type(datasource) == str:  # Fetch the data source in case we need to read the file type
+                assert datasource != ""
+                datasource = client.modela.DataSource(namespace=namespace, name=datasource)
+                assert not datasource.default_resource
+
+            file_type = datasource.spec.Flatfile.FileType
 
         if data_file is not None:
             with open(data_file, 'r') as f:
@@ -109,12 +124,19 @@ class Dataset(Resource):
 
         if gen_datasource:
             datasource = client.modela.DataSource(namespace=namespace, name=name + "-source", version=version,
-                                                  infer_bytes=data_bytes, target_column=target_column)
+                                                  infer_bytes=data_bytes, target_column=target_column, task_type=task_type)
             datasource.submit(replace=True)
 
         self.spec.DatasourceName = datasource.name
         self.spec.Resources = workload
-        self.spec.Task = task_type
+        self.spec.Task = task_type or TaskType.BinaryClassification
+        if not task_type:
+            print("WARNING: No task type was specified. Defaulting to binary classification")
+        elif task_type != datasource.spec.Task:
+            print("WARNING: Dataset task type does not match the task type of its Data Source (you: %s, it: %s)" %
+                  (task_type.name, datasource.spec.Task.name))
+
+        self.spec.Fast = fast
         if sample is not None:
             self.spec.Sample = sample
 
@@ -127,6 +149,7 @@ class Dataset(Resource):
 
     @property
     def status(self) -> DatasetStatus:
+        self.sync()
         return DatasetStatus().copy_from(self._object.status)
 
     def default(self):
@@ -137,10 +160,10 @@ class Dataset(Resource):
     def report(self) -> Report:
         """ Fetch the report associated with the Dataset """
         self.ensure_client_repository()
-        if self._object.status.reportName != "":
-            return self._client.modela.Report(namespace=self.namespace, name=self._object.status.reportName)
-        else:
-            print("Dataset {0} has no report.".format(self.name))
+        if self.status.ReportName == "":
+            raise ValueError("Dataset {0} has no report.".format(self.name))
+
+        return self._client.modela.Report(namespace=self.namespace, name=self.status.ReportName)
 
     @property
     def phase(self) -> DatasetPhase:
@@ -158,17 +181,18 @@ class Dataset(Resource):
         """ Generate a default prediction payload for a model that would be created with the dataset. """
         target = self.datasource.target_column.Name
         return json.dumps(
-            [{col.Name: col.Mean if col.Datatype == DataType.Number else self.datasource.column(col.Name).Enum[0]
-              for col in self.status.Statistics.Columns if col.Name != target}])
+            [{col.Name: self.status.Statistics.column(col.Name).Mean if col.Datatype == DataType.Number else col.Enum[0]
+              for col in self.datasource.spec.Schema.Columns if col.Name != target}])
 
-    def submit_and_visualize(self, replace: bool = False):
+    def submit_and_visualize(self, replace: bool = False, show_progress_bar=True):
         """
         Submit the resource and call visualize().
 
         :param replace: Replace the resource if it already exists on the cluster.
+        :param show_progress_bar: If enabled, the visualization will render a progress bar indicating the study progress.
         """
         self.submit(replace)
-        self.visualize()
+        self.visualize(show_progress_bar)
 
     def visualize(self, show_progress_bar=True):
         """
@@ -184,7 +208,6 @@ class Dataset(Resource):
         current_status = self.status
         try:
             while True:
-                self.sync()
                 if current_status == self.status:
                     time.sleep(0.1)
                     continue
@@ -263,6 +286,11 @@ class Dataset(Resource):
         else:
             raise AttributeError("Object has no client repository")
 
+    async def wait_until_phase(self, phase: DatasetPhase):
+        """ Returns a coroutine which blocks until the specified phase is reached, or the Dataset fails """
+        while self.status.Phase not in (phase, DatasetPhase.Failed):
+            await asyncio.sleep(1/5)
+
     def __repr__(self):
         out = super().__repr__()
         if len(self.status.Statistics.Columns) > 0:
@@ -300,7 +328,7 @@ class DatasetClient:
         ModelaException.process_error(error)
         return False
 
-    def get(self, namespace: str, name: str) -> Union[Dataset, bool]:
+    def get(self, namespace: str, name: str) -> Dataset:
         request = GetDatasetRequest()
         request.namespace = namespace
         request.name = name
@@ -311,7 +339,6 @@ class DatasetClient:
             error = err
 
         ModelaException.process_error(error)
-        return False
 
     def delete(self, namespace: str, name: str) -> bool:
         request = DeleteDatasetRequest()
@@ -326,7 +353,7 @@ class DatasetClient:
         ModelaException.process_error(error)
         return False
 
-    def list(self, namespace: str) -> Union[List[Dataset], bool]:
+    def list(self, namespace: str) -> List[Dataset]:
         request = ListDatasetsRequest()
         request.namespace = namespace
         try:
@@ -336,7 +363,6 @@ class DatasetClient:
             error = err
 
         ModelaException.process_error(error)
-        return False
 
     def profile(self, namespace: str, name: str) -> DatasetProfile:
         request = GetDatasetProfileRequest()
