@@ -1,11 +1,12 @@
-import hashlib
+import socket
+import subprocess
+import time
+from contextlib import closing
 from typing import List
 
 import grpc
 from github.com.metaprov.modelaapi.services.grpcinferenceservice.v1 import grpcinferenceservice_pb2_grpc
 from github.com.metaprov.modelaapi.services.grpcinferenceservice.v1.grpcinferenceservice_pb2 import PredictRequest
-from github.com.metaprov.modelaapi.services.grpcinferenceservice.v1.grpcinferenceservice_pb2_grpc import \
-    GRPCInferenceServiceStub
 
 from modela.ModelaException import ModelaException
 from modela.inference.models import PredictionResult
@@ -18,7 +19,7 @@ class InferenceService:
     prediction proxy.
     """
 
-    def __init__(self, host, port=None, tls_cert=None):
+    def __init__(self, host="localhost", port=None, tls_cert=None, port_forward=False, service_name="", service_namespace=""):
         """
         Connect to the gRPC service.
 
@@ -27,6 +28,19 @@ class InferenceService:
         :param tls_cert: The TLS certificate of the connection, if connecting through ingress. The Secret containing the
             public key of the ingress can be found in the namespace of the Serving Site that hosts the service.
         """
+        if port_forward:
+            with closing(socket.socket(socket.AF_INET, socket.SOCK_STREAM)) as s:
+                s.bind(('', 0))
+                s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
+                port = s.getsockname()[1]
+
+            self.pf_process = subprocess.Popen("kubectl port-forward svc/%s %d:8080 -n %s" %
+                                               (service_name, port, service_namespace),
+                                               shell=True, stderr=subprocess.STDOUT)
+
+            time.sleep(1/4)
+            tls_cert, host = None, "localhost"
+
         if tls_cert:
             with open(tls_cert, 'rb') as f:
                 credentials = grpc.ssl_channel_credentials(f.read())
@@ -52,6 +66,9 @@ class InferenceService:
         return self
 
     def __exit__(self, exc_type, exc_value, traceback) -> None:
+        if hasattr(self, 'pf_process'):
+            self.pf_process.kill()
+
         self.close()
 
     def predict(self,
