@@ -1,31 +1,33 @@
+import asyncio
 import time
+from typing import List, Union
 
 import grpc
 from github.com.metaprov.modelaapi.pkg.apis.training.v1alpha1.generated_pb2 import Study as MDStudy
-from github.com.metaprov.modelaapi.services.study.v1.study_pb2_grpc import StudyServiceStub
 from github.com.metaprov.modelaapi.services.study.v1.study_pb2 import CreateStudyRequest, \
     UpdateStudyRequest, DeleteStudyRequest, GetStudyRequest, ListStudyRequest, \
     GetStudyProfileRequest, CreateStudyProfileRequest, AbortStudyRequest, PauseStudyRequest, ResumeStudyRequest
+from github.com.metaprov.modelaapi.services.study.v1.study_pb2_grpc import StudyServiceStub
 from tqdm import tqdm
 
-from modela.Resource import Resource
 from modela.ModelaException import ModelaException, ResourceNotFoundException
-from typing import List, Union
-
+from modela.Resource import Resource
+from modela.common import *
 from modela.data.Dataset import Dataset
 from modela.infra.Lab import Lab
 from modela.infra.VirtualBucket import VirtualBucket
 from modela.training.Model import Model
-from modela.training.common import TaskType, _StudyPhaseToProgress
-from modela.training.models import *
+from modela.training.common import *
+from modela.training.models import ModelSearch, FeatureEngineeringSearch, BaselineSettings, Ensemble, Training, \
+    Interpretability, StudySchedule, NotificationSettings, StudySpec, StudyStatus, DataLocation
 
 
 class Study(Resource):
     def __init__(self, item: MDStudy = MDStudy(), client=None, namespace="", name="", version=Resource.DefaultVersion,
                  dataset: Union[str, Dataset] = "",
-                 lab: Union[ObjectReference, Lab, str] = "default-lab",
+                 test_dataset: Union[str, Dataset] = "",
+                 lab: Union[ObjectReference, Lab, str] = "modela-lab",
                  bucket: Union[VirtualBucket, str] = None,
-                 task_type: TaskType = None,
                  objective: Metric = None,
                  search: ModelSearch = None,
                  fe_search: FeatureEngineeringSearch = None,
@@ -34,9 +36,10 @@ class Study(Resource):
                  trainer_template: Training = None,
                  interpretability: Interpretability = None,
                  schedule: StudySchedule = None,
-                 notification: NotificationSetting = None,
+                 notification: NotificationSettings = None,
                  garbage_collect: bool = True,
                  keep_best_models: bool = True,
+                 fast: bool = False,
                  timeout: int = 600,
                  template: bool = False):
         """
@@ -47,10 +50,11 @@ class Study(Resource):
         :param version: The version of the resource.
         :param dataset: If specified as a string, the SDK will attempt to find a Dataset resource with the given name.
             If specified as a Dataset object, or if one was found with the given name, it will be used in the Study.
-        :param lab: The object reference, Lab object, or lab name under the default-tenant for which all Study-related
+        :param test_dataset: If specified, the Dataset will be used as the test dataset and the Dataset specified by
+            the `dataset` field will be used solely as the training dataset
+        :param lab: The object reference, Lab object, or lab name under the tenant of the resource for which all Study-related
             workloads will be performed under.
         :param bucket: The Bucket object or name of the bucket which will store the Study artifacts
-        :param task_type: The ML task type of the Study
         :param objective: The objective metric relevant to the task type.
         :param search: The search parameters define how many models to sample
         :param fe_search: The feature engineering search parameters of the Study
@@ -59,72 +63,96 @@ class Study(Resource):
         :param ensemble: The ensemble settings for the Study, which if enabled will combine the top estimators of
             the study after the initial model search.
         :param trainer_template: The training template for each model created by the Study.
-        :param interpretability: The interpretability settings for the Study, which when enabled can produce ICE, LIME,
-            and Shap value plots
+        :param interpretability: The interpretability configuration for the Study, which specifies what type of
+            model explainability plots will be generated from Shap values if they are computed
         :param schedule: The schedule for the study to run chronically
         :param notification: The notification settings, which if enabled will forward events about this resource to a notifier.
         :param garbage_collect: If enabled, models which did not move past the testing stage will be garbage collected by
             the system.
         :param keep_best_models: If enabled, the best models from each algorithm will not be garbage collected.
+        :param fast: If enabled, the Study will skip profiling, explaining, and reporting.
         :param timeout: The timeout in seconds after which the Study will fail.
         :param template: If the Study is a template it will not start a search and can only be used as a template for
             other studies.
         """
+        self.default_resource = False
         super().__init__(item, client, namespace=namespace, name=name, version=version)
+        if not self.default_resource:  # Ignore the rest of the constructor; studies are immutable
+            return
+
+        spec = self.spec
         if type(dataset) == Dataset:
-            dataset = dataset.name
-        self._object.spec.datasetName = dataset
+            dataset_name = dataset.name
+        else:
+            dataset_name = dataset
+            dataset = client.modela.Dataset(namespace=namespace, name=dataset)
+            if dataset.default_resource:
+                raise ResourceNotFoundException("Dataset {0} does not exist".format(dataset_name))
+
+        spec.DatasetName = dataset_name
+        spec.Task = dataset.spec.Task
+
+        if test_dataset:
+            if type(test_dataset) == Dataset:
+                test_dataset_name = dataset.name
+            else:
+                test_dataset_name = dataset
+                test_dataset = client.modela.Dataset(namespace=namespace, name=dataset)
+                if test_dataset.default_resource:
+                    raise ResourceNotFoundException("Dataset {0} does not exist".format(dataset_name))
+
+            spec.TrainingTemplate.Split.TrainDataset = dataset_name
+            spec.TrainingTemplate.Split.TestDataset = test_dataset_name
+            spec.TrainingTemplate.Split.Method = DataSplitMethod.TestDataset
 
         if type(lab) == Lab:
             lab = lab.reference
         elif type(lab) == str:
-            lab = ObjectReference(Namespace="default-tenant", Name=lab)
-        self.spec.LabRef = lab
+            lab = ObjectReference(Namespace=client.modela.tenant, Name=lab)
 
         if bucket is not None:
             if type(bucket) == VirtualBucket:
                 bucket = bucket.name
-            self.spec.Location = DataLocation(BucketName=bucket)
-
-        if task_type is not None:
-            self._object.spec.task = task_type.value
+            spec.Location = DataLocation(BucketName=bucket)
 
         if objective is not None:
-            self._object.spec.search.objective = objective.value
-            self.spec.Search.Objective = objective
+            spec.Search.Objective = objective
 
         if search is not None:
-            self.spec.Search = search
+            spec.Search = search
 
         if fe_search is not None:
-            self.spec.FeSearch = fe_search
+            spec.FeSearch = fe_search
 
         if baseline is not None:
-            self.spec.Baseline = baseline
+            spec.Baseline = baseline
 
         if ensemble is not None:
-            self.spec.Ensembles = ensemble
+            spec.Ensembles = ensemble
 
         if trainer_template is not None:
-            self.spec.TrainingTemplate = trainer_template
+            spec.TrainingTemplate = trainer_template
+
+        spec.TrainingTemplate.LabRef = lab
 
         if interpretability is not None:
-            self.spec.Interpretability = interpretability
+            spec.Interpretability = interpretability
 
         if schedule is not None:
-            self.spec.Schedule = schedule
+            spec.Schedule = schedule
 
         if notification is not None:
-            self.spec.Notification = notification
+            spec.Notification = notification
 
         if garbage_collect:
-            self.spec.Gc.CollectAtStudyEnd = True
+            spec.Gc.CollectAtStudyEnd = True
 
         if keep_best_models:
-            self.spec.Gc.KeepOnlyBestModelPerAlgorithm = True
+            spec.Gc.KeepOnlyBestModelPerAlgorithm = True
 
-        self._object.spec.activeDeadlineSeconds = timeout
-        self._object.spec.template = template
+        spec.Fast = fast
+        # spec.ActiveDeadlineSeconds = timeout
+        spec.Template = template
 
     @property
     def spec(self) -> StudySpec:
@@ -132,9 +160,11 @@ class Study(Resource):
 
     @property
     def status(self) -> StudyStatus:
+        self.sync()
         return StudyStatus().copy_from(self._object.status)
 
     def default(self):
+        self.default_resource = True
         StudySpec().apply_config(self._object.spec)
 
     def abort(self):
@@ -149,14 +179,15 @@ class Study(Resource):
         self.ensure_client_repository()
         self._client.resume(self.namespace, self.name)
 
-    def submit_and_visualize(self, replace: bool = False):
+    def submit_and_visualize(self, replace: bool = False, show_progress_bar=True):
         """
         Submit the resource and call visualize().
 
         :param replace: Replace the resource if it already exists on the cluster.
+        :param show_progress_bar: If enabled, the visualization will render a progress bar indicating the study progress.
         """
         self.submit(replace)
-        self.visualize()
+        self.visualize(show_progress_bar)
 
     def visualize(self, show_progress_bar=True):
         """
@@ -170,11 +201,11 @@ class Study(Resource):
         current_status, objective, cv_top, alg_top = None, self.spec.Search.Objective, 0, ""
         try:
             while True:
-                self.sync()
-                if current_status != self.status:
-                    current_status = self.status
+                status = self.status
+                if current_status != status:
+                    current_status = status
                     desc.set_description('Phase: %s | Active Models: %d | Best Algorithm: %s | Best Score: %s' %
-                                         (_StudyPhaseToProgress[self.phase], current_status.Models,
+                                         (StudyPhaseToProgress[self.phase], current_status.Models,
                                           alg_top if alg_top != "" else "[Waiting]",
                                           str(cv_top) if cv_top != 0 else "[Waiting]"))
 
@@ -192,7 +223,7 @@ class Study(Resource):
                         break
                 elif current_status.Phase == StudyPhase.Completed:
                     model = self._client.modela.Model(self.namespace, current_status.BestModel)
-                    if model.phase != ModelPhase.Completed:
+                    if model.phase not in (ModelPhase.Completed, ModelPhase.Live):
                         continue
 
                     desc.close()
@@ -287,13 +318,21 @@ class Study(Resource):
         except KeyboardInterrupt:
             pass
 
+    async def wait_until_phase(self, phase: StudyPhase):
+        """ Returns a coroutine which blocks until the specified phase is reached, or the Study fails """
+        while self.status.Phase not in (phase, StudyPhase.Failed):
+            await asyncio.sleep(1/5)
+
+
     @property
     def models(self) -> List[Model]:
+        """ Get the list of currently active models produced by the Study """
         self.ensure_client_repository()
         return self._client.modela.Models.list(self.namespace, {'study': self.name})
 
     @property
     def best_model(self) -> Model:
+        """ Get the best model selected by the model search process """
         self.ensure_client_repository()
         return self._client.modela.Models.get(self.namespace, self._object.status.bestModel)
 
